using System;
using System.Reflection;
using System.Collections.Generic;
using System.Text;

namespace Commands
{
	public class CommandManager<T> : ICommandManager<T>
	{
		#region ICommandManager Members

		public void Register(T command, object control)
		{
			EventInfo ei = control.GetType().GetEvent("Click");
			if (ei != null)
			{
				this.controlToCommandLookup.Add(control, command);

				if (!this.commandToControlsLookup.ContainsKey(command))
					this.commandToControlsLookup.Add(command, new List<object>());

				this.commandToControlsLookup[command].Add(control);

                if (!this.commandToEnabledLookup.ContainsKey(command))
                    this.commandToEnabledLookup[command] = true;

				ei.AddEventHandler(control, new EventHandler(control_Click));
			}
		}

		void control_Click(object sender, EventArgs e)
		{
			if (!this.controlToCommandLookup.ContainsKey(sender))
				return;

			T command = this.controlToCommandLookup[sender];
            this.ActivateCommand(command);
		}

        void ActivateCommand(T command)
        {
  			if (!this.commandToHandlersLookup.ContainsKey(command))
				return;
            if (!this.commandToEnabledLookup[command])
                return;

			foreach (CommandHandler<T> handler in this.commandToHandlersLookup[command])
				handler.Invoke();
      }

		public ICommand<T> this[T command]
		{
			get
			{
				Command<T> cmd = new Command<T>();
				cmd.HandlerRegistered += delegate(CommandHandler<T> handler) { this.Register(command, handler); };
				cmd.ControlRegistered += delegate(object control) { this.Register(command, control); };
                cmd.EnabledRequested += delegate(object sender, EventArgs e) { this.SetEnabled(command, true); };
                cmd.DisabledRequested += delegate(object sender, EventArgs e) { this.SetEnabled(command, false); };
                cmd.ActivateRequested += delegate(object sender, EventArgs e) { this.ActivateCommand(command); }; 

				return cmd;
			}
		}


		public void Register(T command, CommandHandler<T> handler)
		{
			if (!this.commandToHandlersLookup.ContainsKey(command))
				this.commandToHandlersLookup.Add(command, new List<CommandHandler<T>>());

            if (!this.commandToEnabledLookup.ContainsKey(command))
                this.commandToEnabledLookup[command] = true;

			this.commandToHandlersLookup[command].Add(handler);
		}

		public void Enable(T command)
		{
			this.SetEnabled(command, true);
		}

		public void Disable(T command)
		{
			this.SetEnabled(command, false);
		}

		void SetEnabled(T command, bool value)
		{
			if (!this.commandToControlsLookup.ContainsKey(command))
				return;

            this.commandToEnabledLookup[command] = value;

			foreach (object c in this.commandToControlsLookup[command])
			{
				PropertyInfo pi = c.GetType().GetProperty("Enabled");
				if (pi != null)
					pi.SetValue(c, value, null);
			}
		}

		#endregion

		private Dictionary<T, IList<CommandHandler<T>>> commandToHandlersLookup = new Dictionary<T, IList<CommandHandler<T>>>();
		private Dictionary<object, T> controlToCommandLookup = new Dictionary<object, T>();
		private Dictionary<T, IList<object>> commandToControlsLookup = new Dictionary<T, IList<object>>();
        private Dictionary<T, bool> commandToEnabledLookup = new Dictionary<T, bool>();
	}
}
